import DISNET.services.ServiceDisnet as ServiceDisnet
import DISNET.Constants as Constants
import DISNET.objects.DiseaseObject as DiseaseObject
import DISNET.objects.DisnetObject as DisnetObject
import DISNET.objects.GeneObject as GeneObject
import DISNET.objects.VariantObject as VariantObject
import DISNET.objects.ProteinObject as ProteinObject
import DISNET.objects.PathwayObject as PathwayObject
import DISNET.objects.PPISObject as PPISObject
import DISNET.Exceptions as Exceptions

C = Constants.Constants  # easier access to constants


class ServiceBio(ServiceDisnet.ServiceDisnet):
    """Class to instance objects to access methods related to the Biology Layer.
    It inherits from the ServiceDisnet class
    """

    def __init__(self):
        ServiceDisnet.ServiceDisnet.__init__(self)

    def getAllDiseases(self)-> list[DiseaseObject.DiseaseObject]:
        """Method to get all the disease objects from the API.

        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_DISEASES + ""
        responseJson = self.connection(url)
        return self.__getAllDiseases(responseJson)

    def getAllGenes(self)->list[GeneObject.GeneObject]:
        """Method to get all the gene objects from the API.

        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_GENES + ""
        responseJson = self.connection(url)
        return self.__getAllGenes(responseJson)

    def getAllVariants(self)->list[VariantObject.VariantObject]:
        """Method to get all the variant objects from the API.

        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_VARIANTS + ""
        responseJson = self.connection(url)
        return self.__getAllVariants(responseJson)

    def getAllPathways(self) -> list[PathwayObject.PathwayObject]:
        """Method to get all the pathway objects from the API.

        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PATHWAYS + ""
        responseJson = self.connection(url)
        return self.__getAllPathways(responseJson)

    def getAllProteins(self)->list[ProteinObject.ProteinObject]:
        """Method to get all the protein objects from the API.

        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PROTEINS + ""
        responseJson = self.connection(url)
        return self.__getAllProteins(responseJson)

    def getAllPPIS(self)->list[PPISObject.PPISObject]:
        """Method to get all the PPIS from the API.

        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PPIS + ""
        responseJson = self.connection(url)
        return self.__getAllPpis(responseJson)

    """
    def getAll(self, next: str) -> list[DisnetObject.DisnetObject]:
        Method to get all the specified Disnet objects from the API.

        :param next: should be one of the following: DISEASES, GENES, VARIANTS, PATHWAYS, PROTEINS, PPIS
        :raises WrongOptionException: when the options are wrong
        :raises NoResultsFoundException: when no results are found

        if next.upper() == "DISEASES":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_DISEASES + ""
            responseJson = self.connection(url)
            return self.__getAllDiseases(responseJson)
        elif next.upper() == "GENES":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_GENES + ""
            responseJson = self.connection(url)
            return self.__getAllGenes(responseJson)
        elif next.upper() == "VARIANTS":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_VARIANTS + ""
            responseJson = self.connection(url)
            return self.__getAllVariants(responseJson)
        elif next.upper() == "PATHWAYS":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PATHWAYS + ""
            responseJson = self.connection(url)
            return self.__getAllPathways(responseJson)
        elif next.upper() == "PROTEINS":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PROTEINS + ""
            responseJson = self.connection(url)
            return self.__getAllProteins(responseJson)
        elif next.upper() == "PPIS":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PPIS + ""
            responseJson = self.connection(url)
            return self.__getAllPpis(responseJson)
        else:
            raise Exceptions.WrongOptionChosen(
                "Possible inputs: DISEASES, GENES, VARIANTS, PROTEINS, PPIS. You entered " + str(next.upper()))
    """

    def getDiseasesByGeneId(self, id: (str, int)) -> list[DiseaseObject.DiseaseObject]:
        """Method to get all the Diseases specified by the Gene Id

        :param id: id to filter by
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_GENES + C.SLASH + str(id) + C.PATH_DISEASES + ""

        responseJson = self.connection(url)
        returnList = self.__getAllDiseases(responseJson)

        return returnList

    def getDiseasesByVariantId(self, id: (str, int)) -> list[DiseaseObject.DiseaseObject]:
        """Method to get all the Diseases specified by the Variant id

        :param id: id to filter by
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_VARIANTS + C.SLASH + str(id) + C.PATH_DISEASES + ""

        responseJson = self.connection(url)
        returnList = self.__getAllDiseases(responseJson)

        return returnList

    """
    def getGenesBy(self, next: str, id: (str, int)) -> list[GeneObject.GeneObject]:
        Method to get all Genes specified by the "next" param
        
        :param next: should be one of the following: DISEASEID, VARIANTID, PATHWAYID, PROTEINID
        :param id: id to filter by, depends on the "next" parameter
        :raises WrongOptionException: when the options are wrong
        :raises NoResultsFoundException: when no results are found
        

        if next.upper() == "DISEASEID":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_DISEASES + C.SLASH + str(id) + C.PATH_GENES + ""
        elif next.upper() == "VARIANTID":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_VARIANTS + C.SLASH + str(id) + C.PATH_GENES + ""
        elif next.upper() == "PATHWAYID":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PATHWAYS + C.SLASH + str(id) + C.PATH_GENES + ""
        elif next.upper() == "PROTEINID":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PROTEINS + C.SLASH + str(id) + C.PATH_GENES + ""
        else:
            raise Exceptions.WrongOptionChosen(
                "Possible inputs: DISEASEID, VARIANTID, PATHWAYID, PROTEINID. You entered " + str(next.upper()))

        responseJson = self.connection(url)
        returnList = self.__getAllGenes(responseJson)

        return returnList
    """

    def getGenesByDiseaseId(self, id: (str, int)) -> list[GeneObject.GeneObject]:
        """Method to get all Genes specified by the Disease id

        :param id: id to filter by
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_DISEASES + C.SLASH + str(id) + C.PATH_GENES + ""
        responseJson = self.connection(url)
        returnList = self.__getAllGenes(responseJson)

        return returnList

    def getGenesByVariantId(self, id: (str, int)) -> list[GeneObject.GeneObject]:
        """Method to get all Genes specified by the Variant id

        :param id: id to filter by
        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_VARIANTS + C.SLASH + str(id) + C.PATH_GENES + ""
        responseJson = self.connection(url)
        returnList = self.__getAllGenes(responseJson)

        return returnList

    def getGenesByPathwayId(self, id: (str, int)) -> list[GeneObject.GeneObject]:
        """Method to get all Genes specified by the Pathway id

        :param id: id to filter by
        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PATHWAYS + C.SLASH + str(id) + C.PATH_GENES + ""
        responseJson = self.connection(url)
        returnList = self.__getAllGenes(responseJson)

        return returnList

    def getGenesByProteinId(self, id: (str, int)) -> list[GeneObject.GeneObject]:
        """Method to get all Genes specified by the Protein id

        :param id: id to filter by
        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PROTEINS + C.SLASH + str(id) + C.PATH_GENES + ""
        responseJson = self.connection(url)
        returnList = self.__getAllGenes(responseJson)

        return returnList

    """
    def getVariantsBy(self, next: str, id: (str, id)) -> list[VariantObject.VariantObject]:
        Method to get all Variants specified by the "next" param
        :param id: id to filter by
        :raises NoResultsFoundException: when no results are found
        

        if next.upper() == "GENEID":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_GENES + C.SLASH + str(id) + C.PATH_VARIANTS + ""
        elif next.upper() == "DISEASEID":
            url = C.MAIN_PATH + C.PATH_BIO + C.PATH_DISEASES + C.SLASH + str(id) + C.PATH_VARIANTS + ""
        else:
            raise Exceptions.WrongOptionChosen("Possible inputs: GENEID, DISEASEID. You entered " + str(next.upper()))

        responseJson = self.connection(url)
        returnList = self.__getAllVariants(responseJson)

        return returnList
    """

    def getVariantsByGeneId(self, id: (str, id)) -> list[VariantObject.VariantObject]:
        """Method to get all Variants specified by the Gene id

        :param id: id to filter by
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_GENES + C.SLASH + str(id) + C.PATH_VARIANTS + ""

        responseJson = self.connection(url)
        returnList = self.__getAllVariants(responseJson)

        return returnList

    def getVariantsByDiseaseId(self, id: (str, id)) -> list[VariantObject.VariantObject]:
        """Method to get all Variants specified by the Disease id

        :param id: id to filter by
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_DISEASES + C.SLASH + str(id) + C.PATH_VARIANTS + ""

        responseJson = self.connection(url)
        returnList = self.__getAllVariants(responseJson)

        return returnList

    def getProteinsByGeneID(self, id: (str, int)) -> list[ProteinObject.ProteinObject]:
        """Method to get all Proteins associated to a Gene id

        :param id: id of the gene to filter by
        :raises WrongOptionException: when the options are wrong
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_GENES + C.SLASH + str(id) + C.PATH_PROTEINS
        responseJson = self.connection(url)
        returnList = self.__getAllProteins(responseJson)

        return returnList

    def getPathwaysByGeneID(self, id):
        """Method to get all Pathways associated to a Gene id

        :param id: id of the gene to filter by
        :raises WrongOptionException: when the options are wrong
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_GENES + C.SLASH + str(id) + C.PATH_PATHWAYS
        responseJson = self.connection(url)
        returnList = self.__getAllPathways(responseJson)

        return returnList

    def getDiseaseByDiseaseName(self, name: str, exactMatch: bool) -> list[DiseaseObject.DiseaseObject]:
        """Method to search Diseases by name

        :param name: name of the Disease to look for
        :param exactMatch: true if the name has to fit exactly, false if it just contains the name
        :raises WrongOptionException: when the options are wrong
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_DISEASES + C.QUESTION_MARK_NAME_EQUALS + \
              name.replace(" ", "%20") + C.AMPERSAND_EXACT_MATCH_EQUALS + str(exactMatch)

        responseJson = self.connection(url)
        returnList = self.__getAllDiseases(responseJson)

        return returnList

    def getGeneByGeneID(self, id: (str, int)) -> GeneObject.GeneObject:
        """Method to search a Gene by its id

        :param id: id of the Gene to look for
        :raises WrongOptionException: when the options are wrong
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_GENES + C.SLASH + str(id)

        responseJson = self.connection(url)
        geneJson = responseJson["data"]

        geneId = geneJson["geneId"]
        geneSymbol = geneJson["geneSymbol"]
        geneName = geneJson["geneName"]

        gene = GeneObject.GeneObject(geneId, geneName, geneSymbol)

        return gene

    def getVariantByVariantID(self, id: (str, int)) -> VariantObject.VariantObject:
        """Method to search a Gene by its id

        :param id: id of the Gene to look for
        :raises WrongOptionException: when the options are wrong
        :raises NoResultsFoundException: when no results are found
        """
        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_VARIANTS + C.SLASH + str(id)

        response = self.connection(url)
        variantJson = response["data"]

        variantId = variantJson["variantId"]
        chromosome = variantJson["chromosome"]
        chrposition = variantJson["chrposition"]
        consequence = variantJson["consequence"]

        variant = VariantObject.VariantObject(variantId, chromosome, chrposition, consequence)

        return variant

    def getPathwayByPathwayID(self, id: (str, int)) -> PathwayObject.PathwayObject:
        """Method to search a Pathway by its id

        :param id: id of the Pathway to look for
        :raises WrongOptionException: when the options are wrong
        :raises NoResultsFoundException: when no results are found
        """

        url = C.MAIN_PATH + C.PATH_BIO + C.PATH_PATHWAYS + C.SLASH + str(id)
        response = self.connection(url)
        pathwayJson = response["data"]

        pathwayId = pathwayJson["pathwayId"]
        pathwayName = pathwayJson["pathwayName"]
        pathway = PathwayObject.PathwayObject(pathwayId, pathwayName);

        return pathway

    def __getAllDiseases(self, responseJson: dict) -> list[DiseaseObject.DiseaseObject]:
        """Method to get all the diseases from the response received in other methods

        :param response: json containing the response of the get petition
        """
        returnList = []

        listDiseasesJson = responseJson["data"]
        for diseaseJson in listDiseasesJson:
            diseaseId = diseaseJson["diseaseId"]
            diseaseName = diseaseJson["diseaseName"]
            disease = DiseaseObject.DiseaseObject(diseaseId, diseaseName)
            returnList.append(disease)

        return returnList

    def __getAllGenes(self, responseJson: dict) -> list[GeneObject.GeneObject]:
        """Method to get all the genes from the response received in other methods

        :param response: json containing the response of the get petition
        """
        returnList = []

        listGenesJson = responseJson["data"]
        for geneJson in listGenesJson:
            geneId = geneJson["geneId"]
            geneName = geneJson["geneName"]
            geneSymbol = geneJson["geneSymbol"]
            gene = GeneObject.GeneObject(geneId, geneName, geneSymbol)
            returnList.append(gene)

        return returnList

    def __getAllVariants(self, responseJson: dict) -> list[VariantObject.VariantObject]:
        """Method to get all the Variants from the response received in other methods

        :param response: json containing the response of the get petition
        """
        returnList = []
        listVariantsJson = responseJson["data"]

        for variantJson in listVariantsJson:
            variantId = variantJson["variantId"]
            chromosome = variantJson["chromosome"]
            chrposition = variantJson["chrposition"]
            consequence = variantJson["consequence"]
            variant = VariantObject.VariantObject(variantId, chromosome, chrposition, consequence)
            returnList.append(variant)

        return returnList

    def __getAllPathways(self, responseJson: dict) -> list[PathwayObject.PathwayObject]:
        """Method to get all the Pathways from the response received in other methods

        :param response: json containing the response of the get petition
        """
        returnList = []
        listPathwaysJson = responseJson["data"]

        for pathwayJson in listPathwaysJson:
            pathwayId = pathwayJson["pathwayId"]
            pathwayName = pathwayJson["pathwayName"]
            pathway = PathwayObject.PathwayObject(pathwayId, pathwayName)
            returnList.append(pathway)

        return returnList

    def __getAllProteins(self, responseJson: dict) -> list[ProteinObject.ProteinObject]:
        """Method to get all the proteins from the response received in other methods

        :param response: json containing the response of the get petition
        """
        returnList = []
        listProteinsJson = responseJson["data"]

        for proteinJson in listProteinsJson:
            proteinId = proteinJson["proteinId"]
            protein = ProteinObject.ProteinObject(proteinId)
            returnList.append(protein)

        return returnList

    def __getAllPpis(self, responseJson: dict) -> list[PPISObject.PPISObject]:
        """Method to get all the PPIS from the response received in other methods

        :param response: json containing the response of the get petition
        """
        returnList = []
        listProteinsJson = responseJson["data"]

        for ppisJson in listProteinsJson:
            protein1Id = ppisJson["protein1Id"]
            protein2Id = ppisJson["protein2Id"]

            protein1 = ProteinObject.ProteinObject(protein1Id)
            protein2 = ProteinObject.ProteinObject(protein2Id)

            ppis = PPISObject.PPISObject(protein1, protein2)
            returnList.append(ppis)

        return returnList
